@@ -11,7 +11,8 @@ namespace Sass {
     // The following list of selectors is by increasing specificity:
     extern const unsigned long Specificity_Star;
     extern const unsigned long Specificity_Universal;
-    extern const unsigned long Specificity_Type;
+    extern const unsigned long Specificity_Element;
+    extern const unsigned long Specificity_Base;
     extern const unsigned long Specificity_Class;
     extern const unsigned long Specificity_Attr;
     extern const unsigned long Specificity_Pseudo;
