@@ -111,7 +111,6 @@ namespace Sass {
     }
 
     IncompatibleUnits::IncompatibleUnits(const Number& lhs, const Number& rhs)
-    : lhs(lhs), rhs(rhs)
     {
       msg  = "Incompatible units: '";
       msg += rhs.unit();
@@ -120,6 +119,15 @@ namespace Sass {
       msg += "'.";
     }
 
+    IncompatibleUnits::IncompatibleUnits(const UnitType lhs, const UnitType rhs)
+    {
+      msg  = "Incompatible units: '";
+      msg += unit_to_string(rhs);
+      msg += "' and '";
+      msg += unit_to_string(lhs);
+      msg += "'.";
+    }
+
     AlphaChannelsNotEqual::AlphaChannelsNotEqual(Expression_Ptr_Const lhs, Expression_Ptr_Const rhs, const std::string& op)
     : lhs(lhs), rhs(rhs), op(op)
     {
