@@ -5,6 +5,8 @@
 #include <sstream>
 #include <stdexcept>
 #include "position.hpp"
+#include "ast_fwd_decl.hpp"
+#include "sass/functions.h"
 
 namespace Sass {
 
@@ -38,10 +40,10 @@ namespace Sass {
 
     class InvalidParent : public Base {
       protected:
-        Selector* parent;
-        Selector* selector;
+        Selector_Ptr parent;
+        Selector_Ptr selector;
       public:
-        InvalidParent(Selector* parent, Selector* selector);
+        InvalidParent(Selector_Ptr parent, Selector_Ptr selector);
         virtual ~InvalidParent() throw() {};
     };
 
@@ -60,9 +62,9 @@ namespace Sass {
         std::string fn;
         std::string arg;
         std::string type;
-        const Value* value;
+        const Value_Ptr value;
       public:
-        InvalidArgumentType(ParserState pstate, std::string fn, std::string arg, std::string type, const Value* value = 0);
+        InvalidArgumentType(ParserState pstate, std::string fn, std::string arg, std::string type, const Value_Ptr value = 0);
         virtual ~InvalidArgumentType() throw() {};
     };
 
@@ -136,37 +138,38 @@ namespace Sass {
 
     class IncompatibleUnits : public OperationError {
       protected:
-        const Number& lhs;
-        const Number& rhs;
+        // const Sass::UnitType lhs;
+        // const Sass::UnitType rhs;
       public:
         IncompatibleUnits(const Number& lhs, const Number& rhs);
+        IncompatibleUnits(const UnitType lhs, const UnitType rhs);
         virtual ~IncompatibleUnits() throw() {};
     };
 
     class UndefinedOperation : public OperationError {
       protected:
-        const Expression* lhs;
-        const Expression* rhs;
+        Expression_Ptr_Const lhs;
+        Expression_Ptr_Const rhs;
         const std::string op;
       public:
-        UndefinedOperation(const Expression* lhs, const Expression* rhs, const std::string& op);
+        UndefinedOperation(Expression_Ptr_Const lhs, Expression_Ptr_Const rhs, const std::string& op);
         // virtual const char* errtype() const { return "Error"; }
         virtual ~UndefinedOperation() throw() {};
     };
 
     class InvalidNullOperation : public UndefinedOperation {
       public:
-        InvalidNullOperation(const Expression* lhs, const Expression* rhs, const std::string& op);
+        InvalidNullOperation(Expression_Ptr_Const lhs, Expression_Ptr_Const rhs, const std::string& op);
         virtual ~InvalidNullOperation() throw() {};
     };
 
     class AlphaChannelsNotEqual : public OperationError {
       protected:
-        const Expression* lhs;
-        const Expression* rhs;
+        Expression_Ptr_Const lhs;
+        Expression_Ptr_Const rhs;
         const std::string op;
       public:
-        AlphaChannelsNotEqual(const Expression* lhs, const Expression* rhs, const std::string& op);
+        AlphaChannelsNotEqual(Expression_Ptr_Const lhs, Expression_Ptr_Const rhs, const std::string& op);
         // virtual const char* errtype() const { return "Error"; }
         virtual ~AlphaChannelsNotEqual() throw() {};
     };
